import { format } from 'url';
import { createFile } from './createFile.mjs';
import { wgetAsync } from './wget.mjs';

async function downloadConfig(template, filePath, temporaryFolder) {
    const REPOSITORY_MAP_FILE_NAME = 'repositoryMap.json';
    // TODO parametryzacaj tego url
    const repositoryUrl = `https://raw.githubusercontent.com/SebastianWesolowski/testTemplate/main/template/${template}`;
    const formatterRepositoryFileNameUrl = ({ repository, fileName, }) => {
        const urlObj = new URL(repository || repositoryUrl);
        urlObj.pathname = `${urlObj.pathname}/${fileName}`;
        return format(urlObj);
    };
    try {
        const repositoryMapFileUrl = formatterRepositoryFileNameUrl({ fileName: REPOSITORY_MAP_FILE_NAME });
        console.log(repositoryMapFileUrl);
        return await wgetAsync(repositoryMapFileUrl, temporaryFolder).then(async (content) => {
            await createFile({
                filePath: `${filePath}/${REPOSITORY_MAP_FILE_NAME}`,
                content,
            });
            for (const fileName of JSON.parse(content).fileMap) {
                await wgetAsync(formatterRepositoryFileNameUrl({ fileName }), temporaryFolder).then(async (contentFile) => {
                    await createFile({
                        filePath: `${filePath}/${fileName}`,
                        content: contentFile,
                    });
                });
            }
            return JSON.parse(content).fileMap;
        });
    }
    catch (err) {
        console.error('Błąd podczas pobierania configu z githuba', err);
        throw err; // Przekaż błąd dalej, aby był obsłużony przez wywołującą funkcję
    }
}

export { downloadConfig };
